package de.fraunhofer.iem.dataprovider.kpi.controller

import de.fraunhofer.iem.dataprovider.kpi.dto.KPIResultDto
import de.fraunhofer.iem.dataprovider.kpi.service.KPIService
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.metrics.*
import de.fraunhofer.iem.dataprovider.toolRun.ToolRunService
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import java.util.UUID

@RestController
@RequestMapping("/kpi")
class KPIController(private val kpiService: KPIService, private val toolRunService: ToolRunService) {

    private val logger = getLogger(javaClass)

    @GetMapping("/{id}")
    suspend fun getKPIByID(@PathVariable id: UUID): KPIResultDto {
        logger.info("Get KPI with id $id")
        val kpiEntity = this.kpiService.findKPIById(id) ?: throw Exception("KPI with id $id can not be found")
        return kpiEntity.toResultDto()
    }

    // TODO: Change this to something meaningful
    @GetMapping("/root/repository/{id}")
    suspend fun getRootKPIByRepositoryId(@PathVariable id: UUID): KPIResultDto {
        logger.info("Get root KPI from repository with id $id")
        val repository = this.toolRunService.findRepoByID(id) ?: throw Exception("Repository can not be found")
        val kpiEntity = this.kpiService.findRootKPIByRepository(repository) ?: throw Exception("Root KPI can not be found")
        return kpiEntity.toResultDto()
    }
}