package de.fraunhofer.iem.dataprovider.kpi.model

import de.fraunhofer.iem.dataprovider.kpi.dto.KPIResultChildDto
import de.fraunhofer.iem.dataprovider.kpi.dto.KPIResultDto
import de.fraunhofer.iem.dataprovider.toolRun.model.Repository
import jakarta.persistence.*
import org.hibernate.annotations.JdbcTypeCode
import org.hibernate.type.SqlTypes
import java.sql.Timestamp
import java.util.*

@Entity
@Table(name = "kpi")
class KPI {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @ManyToOne
    @JoinColumn(name = "repository_id")
    var repository: Repository? = null

    @OneToMany(mappedBy = "from", cascade = [CascadeType.MERGE, CascadeType.REMOVE], orphanRemoval = true)
    var hierarchyEdges: MutableSet<KPIHierarchyEdge> = mutableSetOf()

    @Column(name = "value")
    var value: Int? = null

    @Column(name = "isRoot")
    var isRoot: Boolean? = null

    @Column(name = "name")
    var name: String? = null

    @Column(name = "description")
    var description: String? = null

    @JdbcTypeCode(SqlTypes.TIMESTAMP)
    @Column(name = "timestamp")
    var timestamp: Timestamp? = null

    fun addHierarchyEdges(kpiHierarchyEdges: Collection<KPIHierarchyEdge>) {
        kpiHierarchyEdges.forEach {
            this.addHierarchyEdge(it)
        }
    }

    fun addHierarchyEdge(hierarchyEdge: KPIHierarchyEdge) {
        hierarchyEdge.from = this
        this.hierarchyEdges.add(hierarchyEdge)
    }

    fun toResultDto(): KPIResultDto {
        val childrenList = mutableListOf<KPIResultChildDto>()
        for (edge in this.hierarchyEdges) {
            val id = edge.to!!.id!!
            val weight = edge.weight!!
            childrenList.add(KPIResultChildDto(id, weight))
        }
        return KPIResultDto(
                this.id!!,
                this.name!!,
                this.description!!,
                this.isRoot!!,
                childrenList,
                this.repository!!.id!!
        )
    }
}