package de.fraunhofer.iem.dataprovider.repository.controller

import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.metrics.*
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryDto
import de.fraunhofer.iem.dataprovider.toolRun.ToolRunService
import de.fraunhofer.iem.dataprovider.toolRun.model.Repository
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import java.util.UUID

@RestController
@RequestMapping("/repository")
class RepositoryController(private val toolRunService: ToolRunService) {

    private val logger = getLogger(javaClass)

    @GetMapping()
    suspend fun getAllRepositories(): List<RepositoryDto> {
        logger.info("Get all repositories")
        return toolRunService.getAllRepositories().map { repository: Repository -> RepositoryDto(repository.id!!, repository.name!!) }
    }

    @GetMapping("/{id}")
    suspend fun getRepositoryById(@PathVariable id: UUID): Repository {
        logger.info("Get repository with id $id")
        return this.toolRunService.findRepoByID(id) ?: throw Exception("Repository can not be found")
    }
}